#!/usr/bin/env python3
# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

import atexit
import json
import os
import sys
import tempfile
import warnings

from findlibs import find as _find_library

__version__ = "0.7.0"


_here = os.path.join(os.path.dirname(__file__))
_universal = os.path.exists(os.path.join(_here, "_universal"))


if _universal:
    _versions = dict
else:
    _fonts = f"""<?xml version="1.0"?>
<!DOCTYPE fontconfig SYSTEM "fonts.dtd">
<fontconfig>
<dir>{_here}/share/magics/ttf</dir>
</fontconfig>"""

    _fontcfg = tempfile.mktemp("ecmwflibs.xml")
    with open(_fontcfg, "w") as _f:
        print(_fonts, file=_f)

    if "ECMWFLIBS_MAGPLUS" not in os.environ:
        os.environ["FONTCONFIG_FILE"] = os.environ.get(
            "ECMWFLIBS_FONTCONFIG_FILE", _fontcfg
        )
        os.environ["PROJ_LIB"] = os.environ.get(
            "ECMWFLIBS_PROJ_LIB", os.path.join(_here, "share", "proj")
        )
        os.environ["MAGPLUS_HOME"] = os.environ.get("ECMWFLIBS_MAGPLUS_HOME", _here)

    if "ECMWFLIBS_ECCODES" not in os.environ:
        for env in (
            "ECCODES_DEFINITION_PATH",
            "ECCODES_EXTRA_DEFINITION_PATH",
            "ECCODES_EXTRA_SAMPLES_PATH",
            "ECCODES_SAMPLES_PATH",
            "GRIB_DEFINITION_PATH",
            "GRIB_SAMPLES_PATH",
        ):
            if env in os.environ:
                if "ECMWFLIBS_" + env in os.environ:
                    os.environ[env] = os.environ["ECMWFLIBS_" + env]
                    warnings.warn(
                        "ecmwflibs: using provided '{}' set to '{}".format(
                            env, os.environ[env]
                        )
                    )
                else:
                    warnings.warn(
                        (
                            "ecmwflibs: ignoring provided '{}' set to '{}'. "
                            "If you want ecmwflibs to use this environment variable, use ECMWFLIBS_{} instead. "
                            "If you want to use your own ECCODES library, use ECMWFLIBS_ECCODES."
                        ).format(env, os.environ[env], env)
                    )
                    del os.environ[env]

    # This comes *after* the variables are set, so c++ has access to them
    try:
        from ._ecmwflibs import versions as _versions
    except ImportError as e:
        # ImportError: DLL load failed while importing _ecmwflibs: The specified module could not be found.
        warnings.warn(str(e))
        raise


def universal():
    return _universal


def _cleanup():
    try:
        os.unlink(_fontcfg)
    except Exception:
        pass


if not _universal:
    atexit.register(_cleanup)


_MAP = {
    "magics": "MagPlus",
    "magplus": "MagPlus",
    "grib_api": "eccodes",
    "gribapi": "eccodes",
}

EXTENSIONS = {
    "darwin": ".dylib",
    "win32": ".dll",
}


def _lookup(name):
    return _MAP.get(name, name)


def find(name):
    """Returns the path to the selected library, or None if not found."""

    name = _lookup(name)

    if int(os.environ.get("ECMWFLIBS_DISABLED", "0")):
        warnings.warn(f"ECMWFLIBS_DISABLED is set looking for {name}")
        return None

    if int(os.environ.get("ECMWFLIBS_USED_INSTALLED", "0")):
        path = _find_library(name)
        if path is None:
            warnings.warn(f"ECMWFLIBS_USED_INSTALLED did not find {name}")
        else:
            warnings.warn(f"ECMWFLIBS_USED_INSTALLED found {name} at {path}")
        return path

    if _universal:  # Universal version
        path = _find_library(name)
        if path:
            warnings.warn(f"ecmwflibs universal: found {name} at {path}")
        else:
            warnings.warn(f"ecmwflibs universal: cannot find a library called {name}")
        return path

    env = "ECMWFLIBS_" + name.upper()
    if env in os.environ:
        warnings.warn(
            "ecmwflibs: using provided '{}' set to '{}".format(env, os.environ[env])
        )
        return os.environ[env]

    here = os.path.dirname(__file__)
    extension = EXTENSIONS.get(sys.platform, ".so")

    for libdir in [here + ".libs", os.path.join(here, ".dylibs"), here]:
        if not name.startswith("lib"):
            names = ["lib" + name, name]
        else:
            names = [name, name[3:]]

        if os.path.exists(libdir):
            for file in os.listdir(libdir):
                if file.endswith(extension):
                    for name in names:
                        if name == file.split("-")[0].split(".")[0]:
                            return os.path.join(libdir, file)

    return None


def versions():
    """Returns the list of libraries and their version."""
    v = {"ecmwflibs": __version__}
    v.update(_versions())
    return v


def files():
    here = os.path.dirname(__file__)
    for root, dirs, files in os.walk(here):
        for file in files:
            yield os.path.join(root, file).replace(here, "")


def credits():
    """Displays the list of embedded libraries and their copyright
    notices and/or licenses."""
    here = os.path.dirname(__file__)
    with open(os.path.join(here, "copying/list.json")) as f:
        x = json.loads(f.read())

    for k, v in sorted(x.items()):
        print("*" * 80)
        print("name", k)
        print("home", v["home"])
        print("*" * 80)
        print()
        with open(os.path.join(here, v["path"])) as f:
            print(f.read())

    print("*" * 80)


def builds():
    here = os.path.dirname(__file__)
    with open(os.path.join(here, "versions.txt")) as f:
        for d in f.readlines():
            print(d)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import subprocess
import sys

null = open(os.devnull, "w")


def otool(name):
    try:
        return subprocess.check_output(["otool", "-L", name], stderr=null).decode(
            "utf-8"
        )
    except Exception:
        return ""


def process(name, seen, depth=0):
    if name in seen:
        return
    seen.add(name)
    if name.endswith(".dylib"):
        print(f"{' ' * depth}{name}")
        deps = otool(name)
        for line in deps.split("\n"):
            line = line.strip()
            if line.endswith(":"):
                continue
            bits = line.split()
            if len(bits) < 1:
                continue
            lib = bits[0]
            if lib.startswith("@"):
                continue
            if name == lib:
                continue
            process(lib, seen, depth + 2)


for n in sys.argv[1:]:
    process(n, set())



#!/usr/bin/env python3
import pytest
from Magics import macro


def test_climetlab_grib():
    actions = [
        macro.output(
            output_formats=["png"],
            output_name="climetlab_grib_1",
            output_name_first_page_number=False,
            output_width=680,
            page_frame=False,
            page_id_line=False,
            page_x_length=10.0,
            page_y_length=5.555555555555555,
            subpage_x_length=10.0,
            subpage_x_position=0.0,
            subpage_y_length=5.555555555555555,
            subpage_y_position=0.0,
            super_page_x_length=10.0,
            super_page_y_length=5.555555555555555,
        ),
        macro.mmap(
            subpage_lower_left_latitude=33.0,
            subpage_lower_left_longitude=-27.0,
            subpage_map_projection="cylindrical",
            subpage_upper_right_latitude=73.0,
            subpage_upper_right_longitude=45.0,
        ),
        macro.mcoast(
            map_boundaries=True,
            map_coastline_colour="tan",
            map_coastline_land_shade=True,
            map_coastline_land_shade_colour="cream",
            map_grid=False,
            map_grid_colour="tan",
            map_grid_frame=True,
            map_grid_frame_thickness=5,
            map_label=False,
        ),
        macro.mgrib(
            grib_field_position=0,
            grib_file_address_mode="byte_offset",
            grib_input_file_name="climetlab.grib",
        ),
        macro.mcont(
            contour_automatic_setting="climetlab",
            legend=False,
        ),
        macro.mcoast(
            map_grid=False,
            map_grid_frame=True,
            map_grid_frame_thickness=5,
            map_label=False,
        ),
    ]

    print(actions)

    macro.plot(*actions)

    actions = [
        macro.output(
            output_formats=["png"],
            output_name="climetlab_grib_2",
            output_name_first_page_number=False,
            output_width=680,
            page_frame=False,
            page_id_line=False,
            page_x_length=10.0,
            page_y_length=5.555555555555555,
            subpage_x_length=10.0,
            subpage_x_position=0.0,
            subpage_y_length=5.555555555555555,
            subpage_y_position=0.0,
            super_page_x_length=10.0,
            super_page_y_length=5.555555555555555,
        ),
        macro.mmap(
            subpage_lower_left_latitude=33.0,
            subpage_lower_left_longitude=-27.0,
            subpage_map_projection="cylindrical",
            subpage_upper_right_latitude=73.0,
            subpage_upper_right_longitude=45.0,
        ),
        macro.mcoast(
            map_boundaries=True,
            map_coastline_colour="tan",
            map_coastline_land_shade=True,
            map_coastline_land_shade_colour="cream",
            map_grid=False,
            map_grid_colour="tan",
            map_grid_frame=True,
            map_grid_frame_thickness=5,
            map_label=False,
        ),
        macro.mgrib(
            grib_field_position=526,
            grib_file_address_mode="byte_offset",
            grib_input_file_name="climetlab.grib",
        ),
        macro.mcont(
            contour_automatic_setting="climetlab",
            legend=False,
        ),
        macro.mcoast(
            map_grid=False,
            map_grid_frame=True,
            map_grid_frame_thickness=5,
            map_label=False,
        ),
    ]

    print(actions)
    macro.plot(*actions)


@pytest.mark.parametrize("filename", ["climetlab.nc", "test.nc4"])
def test_climetlab_netcdf(filename):
    from Magics import macro

    output = filename.replace(".", "-") + ".png"

    actions = [
        macro.output(
            output_file=output,
            output_width=680,
            page_frame=False,
            page_id_line=False,
            page_x_length=10,
            page_y_length=5.555555555555555,
            subpage_frame=False,
            subpage_x_length=10,
            subpage_x_position=0.0,
            subpage_y_length=5.555555555555555,
            subpage_y_position=0.0,
            super_page_frame=False,
            super_page_x_length=10,
            super_page_y_length=5.555555555555555,
        ),
        macro.mmap(
            subpage_lower_left_latitude=33.0,
            subpage_lower_left_longitude=-27.0,
            subpage_map_projection="cylindrical",
            subpage_upper_right_latitude=73.0,
            subpage_upper_right_longitude=45.0,
        ),
        macro.mcoast(
            map_boundaries=True,
            map_coastline_colour="tan",
            map_coastline_land_shade=True,
            map_coastline_land_shade_colour="cream",
            map_grid=False,
            map_grid_colour="tan",
            map_grid_frame=True,
            map_grid_frame_thickness=5,
            map_label=False,
        ),
        macro.mnetcdf(
            netcdf_filename=filename,
            netcdf_value_variable="t2m",
        ),
        macro.mcont(
            contour_automatic_setting="climetlab",
            legend=False,
        ),
        macro.mcoast(
            map_grid=False,
            map_grid_frame=True,
            map_grid_frame_thickness=5,
            map_label=False,
        ),
    ]

    print(actions)

    macro.plot(*actions)


if __name__ == "__main__":
    test_climetlab_netcdf()
    # test_climetlab_grib()



#!/usr/bin/env python
import json
import os
import re
import socket
import time
import sys

import urllib.error
import urllib.request
try:
    from html2text import html2text
except ImportError:
    html2text = None


def identity(x):
    return x


_HEADERS = {
    "User-Agent": (
        "Mozilla/5.0 (X11; Linux x86_64; rv:125.0) Gecko/20100101 Firefox/125.0"
    ),
    "Accept": "text/html,text/plain,*/*;q=0.8",
    "Accept-Language": "en-US,en;q=0.5",
}


def _local_fallback_path(url):
    base = os.path.basename(url.rstrip("/"))
    if not base:
        return None

    licenses_dir = os.path.join(os.path.dirname(__file__), "licenses")
    for candidate in (base, base.lower()):
        path = os.path.join(licenses_dir, candidate)
        if os.path.exists(path):
            return path
    return None


def fetch_url_text(url, lib_name, timeout=30, attempts=3):
    errors = []

    for attempt in range(1, attempts + 1):
        try:
            req = urllib.request.Request(url, headers=_HEADERS)
            with urllib.request.urlopen(req, timeout=timeout) as response:
                return response.read().decode("utf-8")
        except urllib.error.HTTPError as error:
            body_preview = error.read().decode("utf-8", errors="replace").strip()
            if len(body_preview) > 500:
                body_preview = f"{body_preview[:500]}..."
            details = [
                f"URL: {url}",
                f"Attempt: {attempt}/{attempts}",
                f"HTTP status: {error.code}",
                f"Reason: {error.reason}",
            ]
            if body_preview:
                details.append(f"Response body (first 500 chars):\n{body_preview}")
            errors.append("\n".join(details))
        except (urllib.error.URLError, TimeoutError, socket.timeout, OSError) as error:
            reason = getattr(error, "reason", str(error))
            errors.append(
                f"URL: {url}\n"
                f"Attempt: {attempt}/{attempts}\n"
                f"Network error: {reason}"
            )

        if attempt < attempts:
            time.sleep(attempt)

    fallback_path = _local_fallback_path(url)
    if fallback_path:
        with open(fallback_path, "r", encoding="utf-8") as fallback_file:
            return fallback_file.read()

    raise RuntimeError(
        f"Failed to download license for {lib_name}.\n"
        + "\n\n---\n\n".join(errors)
    )


ENTRIES = {
    "libMagPlus": {
        "home": "https://github.com/ecmwf/magics",
        "copying": "https://raw.githubusercontent.com/ecmwf/magics/develop/LICENSE",
    },
    "libeccodes": {
        "home": "https://github.com/ecmwf/eccodes",
        "copying": "https://raw.githubusercontent.com/ecmwf/eccodes/develop/LICENSE",
    },
    "libnetcdf": {
        "home": "https://github.com/Unidata/netcdf-c",
        "copying": "https://raw.githubusercontent.com/Unidata/netcdf-c/master/COPYRIGHT",
    },
    "libproj": {
        "home": "https://github.com/OSGeo/PROJ",
        "copying": "https://raw.githubusercontent.com/OSGeo/PROJ/master/COPYING",
    },
    "libpixman": {
        "home": "https://gitlab.freedesktop.org/pixman/pixman",
        "copying": "https://gitlab.freedesktop.org/pixman/pixman/-/raw/master/COPYING",
    },
    "libfribidi": {
        "home": "https://github.com/fribidi/fribidi",
        "copying": "https://raw.githubusercontent.com/fribidi/fribidi/master/COPYING",
    },
    "libharfbuzz": {
        "home": "https://github.com/fribidi/fribidi",
        "copying": "https://raw.githubusercontent.com/fribidi/fribidi/master/COPYING",
    },
    "libuuid": {
        "home": "https://github.com/karelzak/util-linux/tree/master/libuuid",
        "copying": "https://raw.githubusercontent.com/karelzak/util-linux/master/libuuid/COPYING",
    },
    "libpango": {
        "home": "https://github.com/GNOME/pango",
        "copying": "https://raw.githubusercontent.com/GNOME/pango/master/COPYING",
    },
    "libsqlite3": {
        "home": "https://sqlite.org/index.html",
        "copying": "Public Domain, see https://sqlite.org/copyright.html",
    },
    "libfontconfig": {
        "home": "https://gitlab.freedesktop.org/fontconfig/fontconfig",
        "copying": "https://gitlab.freedesktop.org/fontconfig/fontconfig/-/raw/main/COPYING",
    },
    "libbz2": {
        "home": "https://gitlab.com/federicomenaquintero/bzip2",
        "copying": "https://gitlab.com/federicomenaquintero/bzip2/-/raw/master/COPYING",
    },
    "libhdf5": {
        "home": "https://github.com/HDFGroup/hdf5",
        "copying": "https://raw.githubusercontent.com/HDFGroup/hdf5/develop/LICENSE",
    },
    "libhdf5_hl": None,  # Assumed to be part of libhdf5 (hl = high level)
    "libpng": {
        "home": "https://github.com/glennrp/libpng",
        "copying": "https://raw.githubusercontent.com/glennrp/libpng/libpng16/LICENSE",
    },
    "libaec": {
        "home": "https://github.com/MathisRosenhauer/libaec",
        "copying": "https://raw.githubusercontent.com/MathisRosenhauer/libaec/v1.1.3/LICENSE.txt",
    },
    "libexpat": {
        "home": "https://libexpat.github.io",
        "copying": "https://raw.githubusercontent.com/libexpat/libexpat/master/expat/COPYING",
    },
    "libz": {
        "home": "https://zlib.net",
        "copying": "https://zlib.net/zlib_license.html",
        "html": True,
    },
    "libcairo": {
        "home": "https://cairographics.org",
        "copying": "https://gitlab.freedesktop.org/cairo/cairo/-/raw/master/COPYING",
    },
    "libjasper": {
        "home": "https://github.com/jasper-software/jasper",
        "copying": "https://raw.githubusercontent.com/jasper-software/jasper/master/LICENSE.txt",
    },
    "libopenjp2": {
        "home": "https://github.com/uclouvain/openjpeg",
        "copying": "https://raw.githubusercontent.com/uclouvain/openjpeg/master/LICENSE",
    },
    # We build libjpeg-turbo from source (drop-in replacement, installs as libjpeg.so.62)
    "libjpeg": {
        "home": "https://github.com/libjpeg-turbo/libjpeg-turbo",
        "copying": "https://raw.githubusercontent.com/libjpeg-turbo/libjpeg-turbo/main/LICENSE.md",
    },
    "libfreetype": {
        "home": "https://gitlab.freedesktop.org/freetype/freetype/",
        "copying": "https://gitlab.freedesktop.org/freetype/freetype/-/raw/master/docs/FTL.TXT",
    },
    "libdatrie": {
        "home": "https://github.com/tlwg/libdatrie",
        "copying": "https://raw.githubusercontent.com/tlwg/libdatrie/master/COPYING",
    },
    "libthai": {
        "home": "https://github.com/tlwg/libthai",
        "copying": "https://raw.githubusercontent.com/tlwg/libthai/master/COPYING",
    },
    "libX11": {
        "home": "https://github.com/mirror/libX11",
        "copying": "https://raw.githubusercontent.com/mirror/libX11/master/COPYING",
    },
    "libxcb": {
        "home": "https://github.com/corngood/libxcb",
        "copying": "https://raw.githubusercontent.com/corngood/libxcb/master/COPYING",
    },
    "libpcre": {
        "home": "https://github.com/vmg/libpcre",
        "copying": "https://raw.githubusercontent.com/vmg/libpcre/master/LICENCE",
    },
    "libgraphite2": {
        "home": "https://github.com/silnrsi/graphite",
        "copying": "https://raw.githubusercontent.com/silnrsi/graphite/master/COPYING",
    },
    "libffi": {
        "home": "https://github.com/libffi/libffi",
        "copying": "https://raw.githubusercontent.com/libffi/libffi/master/LICENSE",
    },
    "libtiff": {
        "home": "https://gitlab.com/libtiff/libtiff",
        "copying": "https://gitlab.com/libtiff/libtiff/-/raw/master/LICENSE.md",
    },
    # See also https://www.gnu.org/software/gettext/manual/html_node/Discussions.html
    # intl(gettext) is GPL while libintl is LGPL
    "libintl": {
        "home": "https://www.gnu.org/software/gettext/manual/html_node/Licenses.html",
        "copying": "https://www.gnu.org/licenses/lgpl-3.0.txt",
        "html": True,
    },
    # See also https://www.gnu.org/software/libiconv/
    # iconv is GPL while libiconv is LGPL
    "libiconv": {
        "home": "https://www.gnu.org/software/libiconv/",
        "copying": "https://www.gnu.org/licenses/lgpl-3.0.txt",
    },
    "libglib": {
        "home": "https://gitlab.gnome.org/GNOME/glib",
        "copying": "https://gitlab.gnome.org/GNOME/glib/-/raw/main/COPYING",
    },
    "libbrotli": {
        "home": "https://github.com/google/brotli",
        "copying": "https://raw.githubusercontent.com/google/brotli/master/LICENSE",
    },
    "libpcre2": {
        "home": "https://github.com/PCRE2Project/pcre2",
        "copying": "https://raw.githubusercontent.com/PCRE2Project/pcre2/main/LICENCE.md",
    },
    "libzstd": {
        "home": "https://github.com/facebook/zstd",
        "copying": "https://raw.githubusercontent.com/facebook/zstd/master/LICENSE",
    },
    # not completely clear what the definitive source for this library is
    "liblzma": {
        "home": "https://github.com/ShiftMediaProject/liblzma",
        "copying": "https://raw.githubusercontent.com/ShiftMediaProject/liblzma/master/COPYING",
    },
    "libcurl": {
        "home": "https://github.com/curl/curl",
        "copying": "https://raw.githubusercontent.com/curl/curl/master/COPYING",
    },
    "libtinyxml2": {
        "home": "https://github.com/leethomason/tinyxml2",
        "copying": "https://raw.githubusercontent.com/leethomason/tinyxml2/master/LICENSE.txt",
    },
}

PATTERNS = {
    r"^libpng\d+$": "libpng",
    r"^libproj(_\d+)+$": "libproj",
}

ALIASES = {
    "libbrotlicommon": "libbrotli",
    "libbrotlidec": "libbrotli",
    "libpangowin32": "libpango",
    "libzlib1": "libz",
    "libeccodes_memfs": "libeccodes",
    "libgobject": "libglib",  # Part of libglib
    "libgmodule": "libglib",  # Part of libglib
    "libgio": "libglib",  # Part of libglib
    "libpangoft2": "libpango",  # Assumed to be part of libpango
    "libpangocairo": "libpango",  # Assumed to be part of libpango
    "libhdf5_hl": "libhdf5",
    # Legacy HDF SZIP license pages were retired; map to libaec.
    "libsz": "libaec",
    "libszip": "libaec",
    # X.Org stack libraries shipped transitively with cairo/pango.
    "libXrender": "libX11",
    "libXdmcp": "libX11",
    "libXau": "libX11",
    "libXext": "libX11",
}

if False:
    for e in ENTRIES.values():
        if isinstance(e, dict):
            copying = e["copying"]
            if copying.startswith("http"):
                urllib.request.urlopen(copying).close()

libs = {}
missing = []
seen = set()

for line in open(sys.argv[1], "r", encoding="utf-8"):
    lib = "-no-regex-"
    lib = line.strip().split("/")[-1]
    lib = lib.split("-")[0].split(".")[0]

    if lib == "":
        continue

    if not lib.startswith("lib"):
        lib = f"lib{lib}"

    for k, v in PATTERNS.items():
        if re.match(k, lib):
            lib = v

    lib = ALIASES.get(lib, lib)

    if lib not in ENTRIES:
        missing.append((lib, line))
        continue

    if lib in seen:
        continue

    seen.add(lib)

    e = ENTRIES[lib]
    if e is None:
        continue

    libs[lib] = dict(path=f"copying/{lib}.txt", home=e["home"])
    copying = e["copying"]

    filtering = identity
    if e.get("html") and html2text is not None:
        filtering = html2text

    if copying.startswith("http://") or copying.startswith("https://"):
        text = fetch_url_text(copying, lib)
        output_lines = filtering(text).split("\n")
    else:
        output_lines = copying.split("\n")

    with open(f"ecmwflibs/copying/{lib}.txt", "w", encoding="utf-8") as f:
        for n in output_lines:
            print(n, file=f)

    with open("ecmwflibs/copying/list.json", "w", encoding="utf-8") as f:
        print(json.dumps(libs), file=f)


assert len(missing) == 0, json.dumps(missing, indent=4, sort_keys=True)



#!/usr/bin/env python
import os
import shutil
import sys

from dlldiag.common import ModuleHeader

VCPKG1 = "C:/vcpkg/installed/{}-windows/bin/{}"
VCPKG2 = "C:/vcpkg/installed/{}-windows/debug/bin/{}"


def scan_module(module, depth, seen):
    name = os.path.basename(module)

    if name in seen:
        return

    if not os.path.exists(module):
        return

    print(" " * depth, module)
    seen[name] = module

    header = ModuleHeader(module)
    cwd = os.path.dirname(module)
    architecture = header.getArchitecture()
    for dll in header.listAllImports():
        # print("DEBUG", dll)
        scan_module((cwd + "/" + dll), depth + 3, seen)
        scan_module(VCPKG1.format(architecture, dll), depth + 3, seen)
        scan_module(VCPKG2.format(architecture, dll), depth + 3, seen)


seen = {}
scan_module(sys.argv[1], 0, seen)

for k, v in seen.items():
    target = sys.argv[2] + "/" + k
    print("Copy", v, "to", target)
    shutil.copyfile(v, target)



import json

import ecmwflibs


def test_versions():
    versions = ecmwflibs.versions()
    print(json.dumps(versions, sort_keys=True, indent=4))

    assert "eccodes" in versions, versions
    assert "magics" in versions, versions
    assert "ecmwflibs" in versions, versions

    for lib in ("eccodes", "magics"):
        print(lib, ecmwflibs.find(lib))

        assert ecmwflibs.find(lib) is not None


# def test_files():
#     for f in ecmwflibs.files():
#         print(f)



#!/usr/bin/env python3
# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

import io
import os
import os.path
import sys

from setuptools import Extension, find_packages, setup


def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return io.open(file_path, encoding="utf-8").read()


version = None
for line in read("ecmwflibs/__init__.py").split("\n"):
    if line.startswith("__version__"):
        version = line.split("=")[-1].strip()[1:-1]

assert version

libdir = os.path.realpath("install/lib")
incdir = os.path.realpath("install/include")
libs = ["eccodes", "MagPlus"]


# https://docs.python.org/3/distutils/apiref.html
ext_modules = [
    Extension(
        "ecmwflibs._ecmwflibs",
        sources=["ecmwflibs/_ecmwflibs.cc"],
        language="c++",
        libraries=libs,
        library_dirs=[libdir],
        include_dirs=[incdir, os.path.join(incdir, "magics")],
        extra_link_args=["-Wl,-rpath," + libdir],
    )
]


def shared(directory):
    result = []
    for path, dirs, files in os.walk(directory):
        for f in files:
            result.append(os.path.join(path, f))
    return result


# Paths must be relative to package directory...
shared_files = ["versions.txt"]
shared_files += [x[len("install/") :] for x in shared("install/share/magics")]
shared_files += [x[len("ecmwflibs/") :] for x in shared("ecmwflibs/share/proj")]
shared_files += [x[len("ecmwflibs/") :] for x in shared("ecmwflibs/etc")]
shared_files += [x[len("ecmwflibs/") :] for x in shared("ecmwflibs/copying")]

if os.name == "nt":
    for n in os.listdir("ecmwflibs"):
        if n.endswith(".dll"):
            shared_files.append(n)

# print(shared_files)

if "--universal" in sys.argv:
    ext_modules = []
    with open("ecmwflibs/_universal", "wt"):
        pass
    shared_files = ["_universal"]

setup(
    name="ecmwflibs",
    version=version,
    author="ECMWF",
    author_email="software.support@ecmwf.int",
    license="Apache 2.0",
    url="https://github.com/ecmwf/ecmwflibs",
    description="Wraps ECMWF tools (experimental)",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    include_package_data=True,
    package_data={"": shared_files},
    install_requires=[
        "findlibs",
    ],
    python_requires=">=3.10",
    zip_safe=True,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3.14",
        "Programming Language :: Python :: Implementation :: CPython",
        "Operating System :: OS Independent",
    ],
    ext_modules=ext_modules,
)
